<?php

/*
CLASSE COPIADA COM MODIFICAÇÕES DA DEPENDENCIA 'TEKNISA/LIBRARIES' VERSÃO: '3.0.24'. NAMESPACE ORIGINAL 'Teknisa\Libs\Util'
*/

namespace PME\Auth\Util;

// use Teknisa\Libs\Exception\CustomException;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
// use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DataSource\Manager\SQL\ManagerImpl;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Service\Crypt;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;

class Utilities {

    const NRORG_FILTER_ZHU_LOG = " AND ORGANIZATION_ID = ";

    const PARAMGERAL  = 'paramgeral';
    const PARAMFILIAL = 'paramfilial';

    const PRODUCT_MENU_NAME = 'product';
    const JSON_EXTENSION = 'json';
    const GENERATED_FILE_NAME = 'modulesVersion';

    const LOGIN_CRYPT_STRING = '1460_loginManagement';

    protected $environment;
    protected $dataSourceManager;
    protected $zhCrypt;

    public function __construct(Environment $environment, ManagerImpl $dataSourceManager) {
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
    }

    /**
     * @param $token
     * @param $requestType
     * @param $data
     * @param $dataSetResultPos
     * @param $privateKey
     * @param $url
     * @param bool $keepConnected
     * @return array
     * @throws \Exception
     */
    public function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected = false){
        set_time_limit(180);
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $privateKey;
        if($keepConnected) {
            $header[] = 'OAuth-KeepConnected: ' . $keepConnected;
        }
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $requestType);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);

        $result = curl_exec($ch);
        $result = json_decode($result, true);
        $error = curl_error($ch);
        curl_close($ch);

        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else {
            return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
        }
    }

    /**
     * @param array[] $filters
     * @param $nameFilter
     * @param bool $remove
     * @return null
     */
    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    /**
     * @param $string
     * @return string
     */
    public static function encrypt($string){
        return md5($string);
    }

    /**
     * @param $password
     * @param bool $isEncrypt
     * @return mixed
     * @throws \Exception
     */
    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true){
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';

        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
        );

        $cryptType = $isEncrypt ? 'GetCriptoPass' : 'GetUnCriptoPass';
        $params = array(
            'rdm'    => 'Bib_html.rdmDBGeralHtm',
            'method' => $cryptType,
            'args'   => $args,
        );

        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }

        return $encryption;
    }

    public static function getCurrentIp() {
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            return $_SERVER['HTTP_CLIENT_IP'];
        }
        if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            return $_SERVER['HTTP_X_FORWARDED_FOR'];
        }

        return $_SERVER['REMOTE_ADDR'];
    }

    public static function createQueryFiltered($filters, $tableAlias = null, $useZhuLogFilters = false)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if (($name != '*' && !empty($values)) || $values == 0) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode($values, "','") . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode($likes, ' OR ');
                        $filterSQL .= "AND ( $filterLikeAll )";

                    } else if ($operator == 'LIKE') {
                        $filterSQL .= "AND $name $operator $values";
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        $customZhuLogFilters = array ();
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    if($useZhuLogFilters && $column == 'ROUTE_LABEL') {
                                        $column = 'ROUTE';
                                    }
                                    if($useZhuLogFilters && isset($customZhuLogFilters[$column])) {
                                        $likes[] = $customZhuLogFilters[$column] . $mappedLikeAllValues . "')";
                                    } else {
                                        $likes[] = "UPPER($column) LIKE UPPER('" . $mappedLikeAllValues . "')";
                                    }
                                }
                                $filterLikeAll = implode($likes, ' OR ');
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                if($useZhuLogFilters && $index == 'ROUTE_LABEL') {
                                    $index = 'ROUTE';
                                }
                                $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }

    /**
     * @return EntityManager
     */
    public static function getEntityManager(){
        return InstanceManager::getInstance()->getService('entityManager');
    }

    public static function zeedhiEncrypt($text, $key = 'libraries') {
        return Crypt::encrypt($text, $key );
    }

    public static function zeedhiDecrypt($text, $key = 'libraries') {
        return Crypt::decrypt($text, $key );
    }

    public static function legacyEncrypt($text, $salt = 'libraries', $padWith = '\0') {
        $encryptedString = trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    str_pad($salt, 32, $padWith),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
        return $encryptedString;
    }

    public static function legacyDecrypt($text, $salt = 'libraries',  $padWith = '\0') {
        $decryptedString = trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                str_pad($salt, 32, $padWith),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
        return $decryptedString;
    }

    /* TEKNISA FUNCTIONS -------------------------------------------------------------------------------------------- */
    public function retornaDataProcessamento(DTO\Request\Row $request, DTO\Response $response) {
        $cdfilial = $request->getRow();
        $dataProcessamento = $this->dataProcessamento($cdfilial[0]);
        $response->addDataSet(new DataSet('processDate', array($this->truncDate($dataProcessamento))));
    }

    public function dataProcessamento($unidade) {
        $dataSet = $this->dataSourceManager->findBy(new FilterCriteria(self::PARAMGERAL),
            array('NRORG' => $this->environment->getNrOrg()));
        $rows = $dataSet->getRows();
        if ($rows[0]['IDUTILDTPROC'] != 'S') {
            $dataProcessamento = date('d/m/Y');
        } else {
            $dataProcessamento = $this->retornaDataProcPorFilial($unidade);
        }
        return $dataProcessamento;
    }

    public function retornaDataProcPorFilial($cdfilial) {
        $filterCriteria = new FilterCriteria(self::PARAMFILIAL);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        if (!empty($rows) && !empty($rows[0]) && $rows[0]['DTPROCESSA']) {
            $dataProcessamento = $rows[0]['DTPROCESSA'];
        }
        else{
            $dataProcessamento = date('d/m/Y');
        }

        return $this->truncDate($dataProcessamento);

    }

    private function truncDate($date) {
        $date = explode(' ', $date);
        return $date[0];
    }

    public function validateUserGroup(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $permission = array('isValid' => $this->permissaoOperadorPorGrupo($row));
        $response->addDataSet(new DataSet('permission', $permission));
    }

    public function permissaoOperadorPorGrupo($row) {
        $filterCriteria = new FilterCriteria('opergrupop');
        $filterCriteria->addCondition('CDOPERADOR', $row['cdoperador']);
        $filterCriteria->addCondition('CDGRUPOPER', $this->retornaGrupoDeOperador($row['idgrupoper']));
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        return isset($rows[0]) ? true : false;
    }

    public function retornaGrupoDeOperador($idautgrupop) {
        $filterCriteria = new FilterCriteria('autgrupop');
        $filterCriteria->addCondition('IDAUTGRUPOP', $idautgrupop);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        return isset($rows[0]) ? $rows[0]['CDGRUPOPER'] : null;

    }

    public function retornaCentralDefault() {
        $filterCriteria = new FilterCriteria('filial');
        $filterCriteria->addCondition('CDFILIAL', $this->environment->getCdFilial());
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());

        $dataSetLogged = $this->dataSourceManager->findBy($filterCriteria);
        $rowsLogged = $dataSetLogged->getRows();
        if($rowsLogged[0]['CDFILCENTPME']) {
            $filterCriteria = new FilterCriteria('filial');
            $filterCriteria->addCondition('CDFILIAL', $rowsLogged[0]['CDFILCENTPME']);
            $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());

            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            $rows = $dataSet->getRows()[0];
        } else {
            $filterCriteria = new FilterCriteria('filial');
            $filterCriteria->addCondition('CDFILCENTPME', $this->environment->getCdFilial());
            $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());

            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            $rows = $dataSet->getRows();
            if(count($rows) > 0) {
                $rows = $rowsLogged[0];
            }
        }

        return $rows;

    }

    /**
     * Converte um string decimal em Float
     * @param $string
     * @return float
     */
    public static function decimalToFloat($string)
    {
        return (float)str_replace(',', '.', $string);
    }

    /**
     * Converte uma string na codificacao iso-8859-1 para utf-8
     * @param $str
     * @return string
     */
    public static function convertISOToUTF8($str)
    {
        $encoding = mb_detect_encoding($str, 'utf-8, iso-8859-1, ascii', true);
        if (strcasecmp($encoding, 'UTF-8') !== 0) {
            $str = iconv($encoding, 'utf-8', $str);
        }

        return $str;
    }

    /**
     * Retira os acentos de uma string.
     *
     * @param string $text string com acentos.
     *
     * @return string
     */
    public static function removeAccents($text) {
        return strtr(utf8_decode($text), utf8_decode("áàãâéêíóôõúüçñÁÀÃÂÉÊÍÓÔÕÚÜÇÑ"), "aaaaeeiooouucnAAAAEEIOOOUUCN");
    }

    /* Cria um arquivo zip com os arquivos passados como parametro
     *
     * @param string $filesUrl Array dos arquivos a serem zipados (caminho completo de cada arquivo)
     * @param string $zipName Nome do arquivo zip a ser criado (caminho completo)
     *
     * @return string caminho completo do zip
     */
    public function zipFiles($filesUrl, $zipname){
        $zip = new \ZipArchive;

        if(is_file($zipname))
            $zip->open($zipname, \ZipArchive::OVERWRITE);
        else
            $zip->open($zipname, \ZipArchive::CREATE);
        foreach ($filesUrl as $file) {
            $filename = explode(DIRECTORY_SEPARATOR, $file);
            $zip->addFile($file, end($filename));
        }

        $caminhoZip = $zip->filename;
        $zip->close();

        return $caminhoZip;
    }

    public static function getProductBasePath($isModulePath = false) {
        if(strpos(__DIR__, 'modules') === false || $isModulePath) {
            return explode('backend', __DIR__)[0];
        }
        return explode('modules', __DIR__)[0];
    }

    public static function downloadAndRemoveFile($fileDir, $fileName) {
        header('Accept-Ranges: none');
        header("Content-Type: text/plain");
        header("Content-Length: " . filesize($fileDir));
        header("Content-Disposition: attachment; filename=" . $fileName);

        readfile($fileDir);
        unlink($fileDir);
        exit;
    }

    public static function getGeneratedFile($ext = self::JSON_EXTENSION, $fileName = self::GENERATED_FILE_NAME) {
        return self::getGeneratedFileDir() . $fileName . '_' . rand(0, 999999) . '.' . $ext;
    }

    public static function getGeneratedFileDir() {
        return __DIR__ . '/../../../../config/';
    }

    public static function getLoginConfig() {
        return json_decode(file_get_contents(__DIR__ . '/../../../../config/config.json'), true);
    }

    public static function getLoginIntegration() {
        return json_decode(file_get_contents(__DIR__ . '/../../../../config/integration.json'), true)[0];
    }

    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }

    public static function getTokenLifeTimeParameter() {
        return InstanceManager::getInstance()->getParameter('tokenLifeTime');
    }

    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }

    public static function getProjectIdParameter() {
        return InstanceManager::getInstance()->getParameter('projectId');
    }

    public static function getUseSQLWebTunningParameter() {
        try {
            return InstanceManager::getInstance()->getParameter('useSQLWebTunning') == 'true';
        } catch(\Exception $e) {
            return true;
        }
    }

    /**
     * @param CustomException $e
     * @return Error
     */
    public static function getErrorObjectFromException(CustomException $e) {
        $error = array (
            'CODE'   => $e->getMessage(),
            'PARAMS' => $e->getParams()
        );
        return new Error(json_encode($error), $e->getCode(), $e->getTraceAsString());
    }
} 