import {
  Metadata, Loader, IEventParam, IDictionary, Http, Datasource, I18n,
} from '@zeedhi/core';
import {
  Button, ModalService, Modal, IButtonEvent, Select, Grid, ITabs,
} from '@zeedhi/common';
import { Cardapio } from '@/components/cardapio/common/Cardapio';
import { default as moment } from 'dayjs';

export class Pme00606AnaliseNutr {
  private modalAnaliseNutr!: Modal;

  private utilities = Loader.getInstance('Utilities');

  public modalController = Loader.getInstance('ModalController');

  public appController = Loader.getInstance('AppController');

  public showNutr = false;

  public filterDates: IDictionary= [];

  public currentPlate: IDictionary = {};

  private nrsequnutr = '';

  public get hasPlateCurrentRow() {
    return !!this.currentPlate.CDPRATO;
  }

  public get lgPlateCardCol() {
    return this.hasPlateCurrentRow ? 6 : 12;
  }

  // infoNutrients
  public proteina = '';

  public lipidio = '';
  
  public carboidrato = '';
  
  public alcool = '';

  private infoNutr: IDictionary = {};

  public vct = '';

  public ndpcal = '';

  // modal control functions
  public async createModal() {
    this.modalAnaliseNutr = await ModalService.createFromJson('modals/pme00606_analise_nutricional', true);
  }

  public cancelModal() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    cardapio.setValueAction(false);
    this.modalAnaliseNutr.hide();
  }
  
  public async openModalAnalNutr() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    cardapio.setValueAction(true);
    
    if (this.modalAnaliseNutr) {
      cardapio.setValueAction(false);
      this.modalAnaliseNutr.show();
    } else {
      this.createModal().then(() => {
        this.modalAnaliseNutr.show();
      });
    }
  }

  public showModal({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    if (cardapio.filterData.NRCARDPME !== undefined) {
      this.procedureNutrCard(cardapio.filterData);
      this.setNutrInfo();
      this.openModalAnalNutr();
    } else {
      this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
    }
  }

  public fullscreen({ component }: IButtonEvent) {
    this.modalAnaliseNutr.fullscreen = !this.modalAnaliseNutr.fullscreen;
    this.modalAnaliseNutr.cssClass = this.modalAnaliseNutr.fullscreen ? 'fullscreen-fill-content' : 'modal-analise-nutricional';
    component.iconName = (this.modalAnaliseNutr.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }

  // filter functions
  public getFilterDates() {
    const datasourceDatas: IDictionary[] = [];
    const date = Metadata.getInstance('dateNutr') as Select;
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    this.filterDates = [];
    const dateFormat: string | undefined = localStorage.getItem('DATEFORMAT') || undefined;
    let initialDate = moment(cardapio.filterData.initialDate, 'DD/MM/YYYY');
    const finalDate = moment(cardapio.filterData.finalDate, 'DD/MM/YYYY');

    datasourceDatas.push({ name: initialDate.format(`${dateFormat} ddd`), value: initialDate.format('DD/MM/YYYY') });
    while (this.utilities.isSameOr(initialDate, finalDate, 'before')) {
      initialDate = initialDate.add(1, 'days');
      if (initialDate <= finalDate) datasourceDatas.push({ name: initialDate.format(`${dateFormat} ddd`), value: initialDate.format('DD/MM/YYYY') });
    }
    
    date.datasource.updateData(datasourceDatas).then(() => {
      const selectedDate: IDictionary = { ...cardapio.getCurrentSelectedRegister() };
      if (selectedDate.DTCARDIAPME) {
        date.setValue(selectedDate.DTCARDIAPME.substring(0, 10));
      } else date.setValue(date.datasource.data[0].value);
    });
  }

  public changeDate({ component }: IButtonEvent) {
    const date = Metadata.getInstance('dateNutr') as Select;
    let momentDate = moment(date.value, 'DD/MM/YYYY');
    if (component.name === 'rightButton') momentDate = moment(date.value, 'DD/MM/YYYY').add(1, 'days');
    else momentDate = moment(date.value, 'DD/MM/YYYY').subtract(1, 'days');

    const hasDate = date.datasource.data.find((data: IDictionary) => data.value === momentDate.format('DD/MM/YYYY'));
    if (hasDate !== undefined) date.setValue(momentDate.format('DD/MM/YYYY'));
  }

  public filterPlatesNutr() {
    const date = Metadata.getInstance('dateNutr') as Select;
    const gridPlates = Metadata.getInstance('plateGrid') as Grid;
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    
    const cdfilial = cardapio.filterData.aggregated ? cardapio.filterData.aggregated.CDFILIAL : cardapio.filterData.cdfilcentpme;
    const conditions = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: cardapio.filterData.cdfilcentpme,
      },
      {
        name: 'P_CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
      {
        name: 'P_CDSERVFILI',
        operator: '=',
        value: cardapio.filterData.service.CDSERVFILI,
      },
      {
        name: 'P_NRCARDPME',
        operator: '=',
        value: cardapio.filterData.NRCARDPME,
      },
      {
        name: 'P_DTCARDAPIO',
        operator: '=',
        value: date.value,
      },
      {
        name: 'P_NRSEQUNUTR',
        operator: '=',
        value: this.nrsequnutr,
      },
    ];
    
    gridPlates.datasource.filter = conditions;
    gridPlates.datasource.get().then(() => {
      const data = gridPlates.datasource.data;
      gridPlates.datasource.currentRow = (data.length > 0) ? data[0] : {};
    });
    const tab = Metadata.getInstance<ITabs>('nutrCardTab');
    if (tab.activeTab === 0) this.filterNutrCard();
  }

  public filterNutr() {
    const tab = Metadata.getInstance<ITabs>('nutrCardTab');
    if (tab.activeTab === 1) {
      const nutrGrid = Metadata.getInstance('nutrGrid') as Grid;
      const conditions = this.getCommonFilter();

      conditions.push(
        {
          name: 'P_CDPRATO',
          operator: '=',
          value: this.currentPlate.CDPRATO ? this.currentPlate.CDPRATO : null,
        },
        {
          name: 'P_NRSEQPRATNUT',
          operator: '=',
          value: this.currentPlate.NRSEQPRACPME ? this.currentPlate.NRSEQPRACPME : null,
        },
      );
      nutrGrid.datasource.filter = conditions;
      nutrGrid.datasource.get();

      this.calculateInfoNutr();
    }
  }

  public filterNutrCard() {
    const nutrCardGrid = Metadata.getInstance('nutrCardGrid') as Grid;
    const conditions = this.getCommonFilter();
    const gridPlates = Metadata.getInstance('plateGrid') as Grid;
    const selectedPlates = gridPlates.selectedRows;
    let plates = '';
    selectedPlates.forEach((plate) => {
      plates += `${plate.CDPRATO}_`;
    });
    
    conditions.push(
      {
        name: 'P_CDPRATO',
        operator: '=',
        value: selectedPlates.length > 0 ? plates : 'T',
      },
      {
        name: 'P_NRSEQPRATNUT',
        operator: '=',
        value: 'T',
      },
    );
    nutrCardGrid.datasource.filter = conditions;
    nutrCardGrid.datasource.get();

    this.calculateInfoNutr();
  }

  private getCommonFilter() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const date = Metadata.getInstance('dateNutr') as Select;
    const cdfilial = cardapio.filterData.aggregated ? cardapio.filterData.aggregated.CDFILIAL : cardapio.filterData.cdfilcentpme;
    const conditions = [
      {
        name: 'P_NRSEQUNUTR',
        operator: '=',
        value: this.nrsequnutr ? this.nrsequnutr : null,
      },
      {
        name: 'P_CDFILINUTR',
        operator: '=',
        value: cdfilial,
      },
      {
        name: 'P_CDSERVNUTR',
        operator: '=',
        value: cardapio.filterData.service.CDSERVFILI,
      },
      {
        name: 'P_DTREFENUTR',
        operator: '=',
        value: date.value ? date.value : null,
      },
      {
        name: 'P_CDDIETNUTR',
        operator: '=',
        value: 'XXXX',
      },
    ];
    return conditions;
  }

  public changeTab() {
    const tab = Metadata.getInstance<ITabs>('nutrCardTab');
    if (tab.activeTab === 1) this.filterNutr();
    else this.filterNutrCard();
  }
  
  // domains functions
  private procedureNutrCard(filter: IDictionary) {
    Http.post('/procedureNutrCard', { requestType: 'Row', row: filter }).then((response) => {
      if (response.data.dataset.nrseqnutr) {
        this.nrsequnutr = response.data.dataset.nrseqnutr[0];
        this.getFilterDates();
      }
    });
  }

  private calculateInfoNutr() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const cdfilial = cardapio.filterData.aggregated ? cardapio.filterData.aggregated.CDFILIAL : cardapio.filterData.cdfilcentpme;
    const date = Metadata.getInstance('dateNutr') as Select;

    let plate = '';
    const tab = Metadata.getInstance<ITabs>('nutrCardTab');
    if (tab.activeTab === 1) plate = this.currentPlate.CDPRATO ? this.currentPlate.CDPRATO : null;
    else {
      const gridPlates = Metadata.getInstance('plateGrid') as Grid;
      const selectedPlates = gridPlates.selectedRows;

      selectedPlates.forEach((item: IDictionary) => {
        plate += `${item.CDPRATO}_`;
      });
    }

    if (plate === null || plate === '') plate = 'T';

    const row = {
      CDPRATO: plate,
      NRSEQUNUTR: this.nrsequnutr,
      CDFILINUTR: cdfilial,
      CDSERVNUTR: cardapio.filterData.service.CDSERVFILI,
      DTREFENUTR: date.value ? date.value : null,
      CDDIETNUTR: 'XXXX',
      CDCARBOHIDRA: this.infoNutr.CDCARBOHIDRA ? this.infoNutr.CDCARBOHIDRA : null,
      CDPROTEINA: this.infoNutr.CDPROTEINA ? this.infoNutr.CDPROTEINA : null,
      CDALCOOL: this.infoNutr.CDALCOOL ? this.infoNutr.CDALCOOL : null,
      CDLIPIDIOS: this.infoNutr.CDLIPIDIOS ? this.infoNutr.CDLIPIDIOS : null,
    };
    Http.post('calculateInfoNutr', { requestType: 'Row', row }).then((response) => {
      if (response.data.dataset) {
        const calculo = response.data.dataset.result.calculo;
        let ndpcal = response.data.dataset.result.ndpcal;
        ndpcal = ndpcal.length > 0 ? ndpcal[0].VRCALORNUTR : 0;
        
        this.ndpcal = `${I18n.translate('pme::ndpcal')} - ${ndpcal.toFixed(3).replace('.', ',').replace(/(\d)(?=(\d{3})+,)/g, '$1.')}%`;

        let proteina = 0;
        let lipidio = 0;
        let carboidrato = 0;
        let alcool = 0;

        let vct = 0;
        let calc = 0;
        
        calculo.forEach((item: IDictionary) => {
          if (item.CDNUTRCALC === this.infoNutr.CDPROTEINA) {
            calc = item.QTNUTRIENTE * 4;
            proteina = calc * 100;
          } else if (item.CDNUTRCALC === this.infoNutr.CDLIPIDIOS) {
            calc = item.QTNUTRIENTE * 9;
            lipidio = calc * 100;
          } else if (item.CDNUTRCALC === this.infoNutr.CDCARBOHIDRA) {
            calc = item.QTNUTRIENTE * 4;
            carboidrato = calc * 100;
          } else if (item.CDNUTRCALC === this.infoNutr.CDALCOOL) {
            calc = item.QTNUTRIENTE * 7;
            alcool = calc * 100;
          }
          vct += calc;
        });
        
        this.vct = `${I18n.translate('pme::vct')} - ${vct.toFixed(3).replace('.', ',').replace(/(\d)(?=(\d{3})+,)/g, '$1.')}%`;
        vct = vct === 0 ? 1 : vct;

        this.proteina = `${I18n.translate('pme::proteina')} - ${(proteina / vct).toFixed(3).replace('.', ',').replace(/(\d)(?=(\d{3})+,)/g, '$1.')}%`;
        this.lipidio = `${I18n.translate('pme::lipidio')} - ${(lipidio / vct).toFixed(3).replace('.', ',').replace(/(\d)(?=(\d{3})+,)/g, '$1.')}%`;
        this.carboidrato = `${I18n.translate('pme::carboidrato')} - ${(carboidrato / vct).toFixed(3).replace('.', ',').replace(/(\d)(?=(\d{3})+,)/g, '$1.')}%`;
        this.alcool = `${I18n.translate('pme::alcool')} - ${(alcool / vct).toFixed(3).replace('.', ',').replace(/(\d)(?=(\d{3})+,)/g, '$1.')}%`;
      }
    });
  }

  public onChangeCurrentPlate({ component }: IEventParam<Datasource>) {
    this.currentPlate = component.currentRow;
    this.filterNutr();
  }

  public deleteNutrTrab() {
    Http.post('/deleteNutrTrab', { requestType: 'Row', row: [this.nrsequnutr] });
  }

  public setNutrInfo() {
    this.proteina = `${I18n.translate('pme::proteina')} - `;
    this.lipidio = `${I18n.translate('pme::lipidio')} - `;
    this.carboidrato = `${I18n.translate('pme::carboidrato')} - `;
    this.alcool = `${I18n.translate('pme::alcool')} - `;
    Http.get('nutrParamgeral').then((response) => {
      if (response.data.data) {
        this.infoNutr = response.data.data[0];
        this.calculateInfoNutr();
      }
    });
  }
}
