import {
  Loader, Metadata, Config, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Button, CheckboxMultiple, Select,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme08100CustoRealPeriodoController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');

  private impTipoProduto = {
    ESTOCAVEIS: 'S',
    NAOESTOCAVEIS: 'S',
  };
  
  public clearFields() {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    unidade.setValue('');
  }
  
  public setFilterAgreg({ component }: IEventParam<Select>) {
    const nutri = Metadata.getInstance<Select>('NUTRICIONISTA');
    const conditions = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'P_CDOPERADOR',
        operator: '=',
        value: nutri.value.length > 0 ? nutri.value.join('_') : null,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      row.reportFormat = component.name;
      this.utilities.activeLoadingButton(component);
      this.reportController.generateReport('report/custoRealPeriodo', row, row.CD_REPORT_CODE).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.CDTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.CDTIPOCUSTO : null;
    row.NMTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.NMTIPOCUSTO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.ESTOCAVEIS = this.impTipoProduto.ESTOCAVEIS;
    row.NAOESTOCAVEIS = this.impTipoProduto.NAOESTOCAVEIS;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
    row.AGRUPMASK = this.utilities.getReportMaskNivel(mask, row.NIVEL);
  }

  private formIsValid(form: IDictionary) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      const dtDe = row.DTDE;
      const dtAte = row.DTATE;

      if (this.utilities.dateIsValid(dtDe, dtAte)) valid = true;
      else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'red');
    }
    return valid;
  }

  public setTipoProduto({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('ESTOCAVEIS')) this.impTipoProduto.ESTOCAVEIS = 'S';
      else this.impTipoProduto.ESTOCAVEIS = 'N';

      if (items.includes('NAOESTOC')) this.impTipoProduto.NAOESTOCAVEIS = 'S';
      else this.impTipoProduto.NAOESTOCAVEIS = 'N';
    }
  }
}
