import {
  Loader, Metadata, IEventParam, Http, IDictionary, Config, I18n,
} from '@zeedhi/core';
import {
  Select, Form, GridEditable, IDate, TextInput, Button, Grid, IGridEvent, ITabs, Modal, ModalService, CheckboxMultiple, DialogService, SelectMultiple,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
import { default as moment } from 'dayjs';

export class Pme00700ConsPorServicoController {
  public appController = Loader.getInstance('AppController');

  public gridEditController = Loader.getInstance('GridEditableController');

  public utilities = Loader.getInstance('Utilities');

  public modalController = Loader.getInstance('ModalController');
  
  public reportController = Loader.getInstance('ReportController');

  private tipoCons: string|null = null;
  
  private modalExport!: Modal;
  
  private modalLog!: Modal;
  
  private modalCadAut!: Modal;
  
  private modalSubsProd!: Modal;
  
  public selectedLine: IDictionary = {};

  public setFilterServ({ component }: IEventParam<Select>) {
    const unidade: Select = Metadata.getInstance('UNIDADE');
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value || null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public beforeFilterGrids() {
    const dtValidade = Metadata.getInstance<IDate>('DTVALIDADE');
    dtValidade.value = '';

    try {
      const dtExcecao = Metadata.getInstance<IDate>('DTEXCECAO');
      dtExcecao.value = '';
    } catch (ex) {
      // continue regardless of error
    }

    this.setFilterGrids();
  }

  public setFilterGrids() {
    const tabs = Metadata.getInstance<ITabs>('tabs');
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const unidade = formFiltro.value.UNIDADE;
    const servico = formFiltro.value.SERVICO;

    this.gridEditController.newRow = false;
    this.gridEditController.isInlineEditing = false;

    if (unidade === null || servico === null) this.disableAddButtons(tabs.activeTab, true);
    else this.disableAddButtons(tabs.activeTab, false);

    if (tabs.activeTab === 0) {
      const dtValidade = Metadata.getInstance<IDate>('DTVALIDADE');
      if (formFiltro.validate()) {
        if (dtValidade.value) this.setFilterGridCons();
        else this.filterDefaultDates(dtValidade, '');
      } else {
        const gridCons = Metadata.getInstance<GridEditable>('CONSPADRAO');
        gridCons.datasource.data = [];
        dtValidade.value = '';
      }
    } else if (tabs.activeTab === 1) {
      const dtExcecao = Metadata.getInstance<IDate>('DTEXCECAO');
      
      if (dtExcecao.value) this.setFilterGridExcecao();
      else this.filterDefaultDates(dtExcecao, '9');
    } else this.setFilterGridSemCons();
  }

  private filterDefaultDates(fieldDate: IDate, idTipo = '') {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const unidade = formFiltro.value.UNIDADE;
    const servico = formFiltro.value.SERVICO;
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade || null,
      },
      {
        name: 'CDSERVICO',
        operator: '=',
        value: servico || null,
      },
      {
        name: 'DATAATUAL',
        operator: '=',
        value: moment().format('DD/MM/YYYY'),
      },
      {
        name: 'IDTIPOCONPME',
        operator: '=',
        value: idTipo || null,
      },
    ];

    Http.post('find/vigenciaAtualConsumo', { requestType: 'FilterData', filter }).then((response) => {
      let date = '';
      if (response.data.data.length > 0) {
        date = response.data.data[0].DTCONSERVPME;
      }
      fieldDate.value = date;
      if (idTipo === '9') this.setFilterGridExcecao();
      else this.setFilterGridCons();
    });
  }
  
  public disableAddButtons(activeTab: number | undefined, disable: boolean) {
    if (activeTab === 0) {
      const addButtonCons = Metadata.getInstance<Button>('addButtonCons');
      addButtonCons.disabled = disable;
    } else if (activeTab === 1) {
      const addButtonExc = Metadata.getInstance<Button>('addButtonExcecao');
      addButtonExc.disabled = disable;
    } else {
      const addButtonSemCons = Metadata.getInstance<Button>('addButtonSemCons');
      addButtonSemCons.disabled = disable;
    }
  }

  public setFilterGridCons() {
    const addButton = Metadata.getInstance<Button>('addButtonCons');
    const gridCons = Metadata.getInstance<GridEditable>('CONSPADRAO');
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');

    const filterCons = this.getCommonFilter();
    const formCons = Metadata.getInstance<Form>('formCons');
    const gridConsPadrao = Metadata.getInstance<GridEditable>('CONSPADRAO');

    let date = formCons.value.DTVALIDADE;
    if (date == null || date.length < 10) date = null;

    filterCons.push(
      {
        name: 'DTCONSERVPME',
        operator: '=',
        value: date,
      },
      {
        name: 'IDTIPOCONPME',
        operator: '=',
        value: formCons.value.DIASEMANACONSUMO || null,
      },
      {
        name: 'IDQTDECONPME',
        operator: '=',
        value: formCons.value.TPCONSUMO || null,
      },
    );
    
    addButton.disabled = true;

    gridCons.datasource.filter = filterCons;
    gridCons.datasource.get().then(() => {
      if (formCons.validate()) addButton.disabled = false;
      if (gridCons.editing) this.disableButtonsGrid(gridCons);
    });
    this.getComensais(unidade.value, servico.value);
    // this.createModalExport(); --------- provavelmente o erro de fechar o modal era pq estava sendo criado duas vezes.
    gridConsPadrao.selectedRows = [];
  }

  public setFilterGridExcecao() {
    const addButton = Metadata.getInstance<Button>('addButtonExcecao');
    const formExcecao = Metadata.getInstance<Form>('formExcecao');
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const gridExcecao = Metadata.getInstance<GridEditable>('GRIDEXCECAO');
    const cadAutButton = Metadata.getInstance<Button>('cadAutButton');
    
    if (formFiltro.validate()) {
      const filter = this.getCommonFilter();
      
      let date = formExcecao.value.DTEXCECAO;
      if (date == null || date.length < 10) date = null;

      filter.push(
        {
          name: 'DTCONSERVPME',
          operator: '=',
          value: date,
        },
        {
          name: 'IDTIPOCONPME',
          operator: '=',
          value: '9',
        },
        {
          name: 'IDQTDECONPME',
          operator: '=',
          value: formExcecao.value.TPCONSUMOEXCECAO || null,
        },
      );
      
      addButton.disabled = true;
      cadAutButton.disabled = false;

      gridExcecao.datasource.filter = filter;
      gridExcecao.datasource.get().then(() => {
        if (formExcecao.validate()) addButton.disabled = false;
        if (gridExcecao.editing) this.disableButtonsGrid(gridExcecao);
      });
    } else {
      const dtExcecao = Metadata.getInstance<IDate>('DTEXCECAO');
      gridExcecao.datasource.data = [];
      dtExcecao.value = '';
    }
  }

  public setFilterGridSemCons() {
    const addButton = Metadata.getInstance<Button>('addButtonSemCons');
    const gridSemCons = Metadata.getInstance<GridEditable>('GRIDSEMCONS');
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    addButton.disabled = true;

    if (formFiltro.validate() && gridSemCons) {
      const filter = this.getCommonFilter();
      
      gridSemCons.datasource.filter = filter;
      gridSemCons.datasource.get().then(() => {
        if (formFiltro.value.UNIDADE && formFiltro.value.SERVICO) addButton.disabled = false;
        if (gridSemCons.editing) this.disableButtonsGrid(gridSemCons);
        this.gridEditController.reload({ gridSemCons });
      });
    } else gridSemCons.datasource.data = [];
  }

  private disableButtonsGrid(grid: IDictionary) {
    if (grid.editing) {
      this.gridEditController.newRow = false;
      // grid.cancelEditedRows();
      // this.gridEditController.onInlineEdit({ component: grid });
      this.gridEditController.cancelEditedRows({ component: grid });
    }
  }

  private getCommonFilter() {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    return [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: formFiltro.value.UNIDADE || null,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: formFiltro.value.SERVICO || null,
      },
    ];
  }

  private getComensais(cdUnidade = '', cdServ = '') {
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdUnidade,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdServ,
      },
    ];

    Http.post('find/consServCard', { requestType: 'FilterData', filter }).then((response) => {
      if (response.data.data.length > 0) {
        const data = response.data.data[0];
        const qtCons = Metadata.getInstance<TextInput>('QTCONSERVPME');
        const day = Metadata.getInstance<Select>('DIASEMANACONSUMO');
        let comensais = 0;

        switch (day.value) {
          case '1': comensais = data.QTCOMPMEDOM || 0; break;
          case '2': comensais = data.QTCOMPMESEG || 0; break;
          case '3': comensais = data.QTCOMPMETER || 0; break;
          case '4': comensais = data.QTCOMPMEQUA || 0; break;
          case '5': comensais = data.QTCOMPMEQUI || 0; break;
          case '6': comensais = data.QTCOMPMESEX || 0; break;
          case '7': comensais = data.QTCOMPMESAB || 0; break;
          case '8': comensais = data.QTCOMPMEFER || 0; break;
          default: comensais = 0; break;
        }
        qtCons.value = comensais;
      }
    });
  }

  public setFilterValidade({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value || null,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: servico.value || null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setVigencia({ row }: IGridEvent) {
    if (row) {
      let field!: IDate;
      if (this.tipoCons === null) field = Metadata.getInstance<IDate>('DTVALIDADE');
      else field = Metadata.getInstance<IDate>('DTEXCECAO');
      field.value = row.DTCONSERVPME;
      
      if (field.name === 'DTVALIDADE') this.setFilterGridCons();
      else this.setFilterGridExcecao();

      this.modalController.closeModal();
    }
  }

  public clearServico() {
    const servico: Select = Metadata.getInstance('SERVICO');
    servico.setValue('');
    const tabs = Metadata.getInstance<ITabs>('tabs');
    if (tabs.activeTab === 0) {
      const dtValidade = Metadata.getInstance<IDate>('DTVALIDADE');
      const qtCons = Metadata.getInstance<TextInput>('QTCONSERVPME');
      dtValidade.value = '';
      qtCons.value = '';
    } else if (tabs.activeTab === 1) {
      const dtExcecao = Metadata.getInstance<IDate>('DTEXCECAO');
      dtExcecao.value = '';
    }
  }
  
  // getElements
  public addRowConsumoPadrao({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formCons = Metadata.getInstance<Form>('formCons');

    if (formFiltro.validate() && formCons.validate()) {
      this.gridEditController.addRow({ component });
      (document.getElementsByName('tabExcecao')[0] as IDictionary).className = 'v-tab v-tab--disabled zd-tabs-tab';
      (document.getElementsByName('tabSemCons')[0] as IDictionary).className = 'v-tab v-tab--disabled zd-tabs-tab';
    }
  }
  
  public addRowExcecao({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formExcecao = Metadata.getInstance<Form>('formExcecao');

    if (formFiltro.validate() && formExcecao.validate()) {
      const cadAutButton = Metadata.getInstance<Button>('cadAutButton');
      this.gridEditController.addRow({ component });
      cadAutButton.disabled = true;
      (document.getElementsByName('tabConsPadrao')[0] as IDictionary).className = 'v-tab v-tab--disabled zd-tabs-tab';
      (document.getElementsByName('tabSemCons')[0] as IDictionary).className = 'v-tab v-tab--disabled zd-tabs-tab';
    }
  }
  
  public addRowDiaSemConsumo({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    if (formFiltro.validate()) {
      this.gridEditController.addRow({ component });
      (document.getElementsByName('tabConsPadrao')[0] as IDictionary).className = 'v-tab v-tab--disabled zd-tabs-tab';
      (document.getElementsByName('tabExcecao')[0] as IDictionary).className = 'v-tab v-tab--disabled zd-tabs-tab';
    }
  }
  
  public cancelEditingConsumoPadrao({ component }: IDictionary) {
    this.gridEditController.cancelEditing({ component });
    (document.getElementsByName('tabExcecao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
    (document.getElementsByName('tabSemCons')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
  }
  
  public cancelEditingExcecao({ component }: IDictionary) {
    const cadAutButton = Metadata.getInstance<Button>('cadAutButton');
    this.gridEditController.cancelEditing({ component });
    cadAutButton.disabled = false;
    (document.getElementsByName('tabConsPadrao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
    (document.getElementsByName('tabSemCons')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
  }
  
  public cancelEditingDiaSemConsumo({ component }: IDictionary) {
    this.gridEditController.cancelEditing({ component });
    (document.getElementsByName('tabConsPadrao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
    (document.getElementsByName('tabExcecao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
  }

  public saveConsPadr({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formCons = Metadata.getInstance<Form>('formCons');
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const rows: IDictionary = grid.getEditedRows();

    if (formFiltro.validate() && formCons.validate() && rows.length > 0) {
      let ctrlEstoq = true;
      let active = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = formFiltro.value.SERVICO;
        e.CDFILIAL = formFiltro.value.UNIDADE;
        e.DTCONSERVPME = formCons.value.DTVALIDADE;
        e.IDTIPOCONPME = formCons.value.DIASEMANACONSUMO;
        e.IDQTDECONPME = formCons.value.TPCONSUMO;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
      });
      
      if (!ctrlEstoq) this.utilities.showAlert('alertEstoq', 'pme::addProdNaoControlaEstoq', 'warning');
      else if (!active) this.utilities.showAlert('alertActive', 'pme::prodInativoUnidade', 'warning');
      else {
        this.callSave('/saveConsPadr', rows, grid);
        (document.getElementsByName('tabExcecao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
        (document.getElementsByName('tabSemCons')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
      }
    }
  }

  public saveExcecao({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formExcecao = Metadata.getInstance<Form>('formExcecao');
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const rows: IDictionary = grid.getEditedRows();
    const cadAutButton = Metadata.getInstance<Button>('cadAutButton');
    
    if (formFiltro.validate() && formExcecao.validate() && rows.length > 0) {
      let ctrlEstoq = true;
      let active = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = formFiltro.value.SERVICO;
        e.CDFILIAL = formFiltro.value.UNIDADE;
        e.DTCONSERVPME = formExcecao.value.DTEXCECAO;
        e.IDTIPOCONPME = '9';
        e.IDQTDECONPME = formExcecao.value.TPCONSUMOEXCECAO;
        if (e.__is_new && e.IDCNTRESTOQ === 'N') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV === 'I') active = false;
      });

      if (!ctrlEstoq) this.utilities.showAlert('alertEstoq', 'pme::addProdNaoControlaEstoq', 'warning');
      else if (!active) this.utilities.showAlert('alertActive', 'pme::prodInativoUnidade', 'warning');
      else {
        this.callSave('/saveConsPadr', rows, grid);
        cadAutButton.disabled = false;
        (document.getElementsByName('tabConsPadrao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
        (document.getElementsByName('tabSemCons')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
      }
    }
  }

  public saveSemCons({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formSemCons = Metadata.getInstance<Form>('formSemCons');
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const rows: IDictionary = grid.getEditedRows();
    
    if (formFiltro.validate() && formSemCons.validate() && rows.length > 0) {
      let isValidRows = true;
      rows.forEach((e: IDictionary) => {
        e.PERCAPITA = e.PERCAPITA ? e.PERCAPITA : 'N';
        e.FIXO = e.FIXO ? e.FIXO : 'N';
        e.CDPRODUTO = e.NMPRODUTO;
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = formFiltro.value.SERVICO;
        e.CDFILIAL = formFiltro.value.UNIDADE;

        if (e.PERCAPITA === 'N' && e.FIXO === 'N') isValidRows = false;
      });

      if (isValidRows) {
        this.callSave('/saveSemCons', rows, grid);
        (document.getElementsByName('tabConsPadrao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
        (document.getElementsByName('tabExcecao')[0] as IDictionary).className = 'v-tab zd-tabs-tab';
      } else this.utilities.showAlert('alertSuccess', 'pme::peloMenosUmTiposSim', 'warning');
    }
  }

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    });
  }

  public setFilterVigencia() {
    if (document.getElementsByName('gridVigencia').length > 0) {
      const gridVigencia = Metadata.getInstance<Grid>('gridVigencia');
      if (gridVigencia) {
        const unidade = Metadata.getInstance<Select>('UNIDADE');
        const servico = Metadata.getInstance<Select>('SERVICO');
        const filter = [
          {
            name: 'CDFILCENTPME',
            operator: '=',
            value: this.appController.getCdCentral(),
          },
          {
            name: 'CDFILIAL',
            operator: '=',
            value: unidade.value || null,
          },
          {
            name: 'CDSERVFILI',
            operator: '=',
            value: servico.value || null,
          },
          {
            name: 'IDTIPOCONPME',
            operator: '=',
            value: this.tipoCons,
          },
        ];
  
        gridVigencia.datasource.filter = filter;
        gridVigencia.datasource.get().then(() => {
          const datasource = gridVigencia.datasource.data;
          datasource.forEach((collumn: IDictionary) => {
            const diaSemana = moment(collumn.DTCONSERVPME, 'DD/MM/YYYY').format('dddd');
            collumn.DIASEMANA = diaSemana;
          });
        });
      }
    }
  }

  public destroyModais() {
    this.modalController.destroyModal();
    this.modalLog.destroy();
  }
  
  public async createModalLog() {
    this.modalLog = await ModalService.createFromJson('modals/pme00708_substituicao_de_produtos', true);
  }
  
  public cancelModalLog() {
    this.modalLog.hide();
  }
  
  public openLogSubsProdutosModal() {
    if (!this.modalLog) {
      this.createModalLog().then(() => {
        this.modalLog.show();
      });
    } else {
      this.modalLog.show();
    }
  }
  
  public openVigenciaModal({ component }: IEventParam<Button>) {
    const modal = this.modalController.getModal();
    if (component.name === 'searchValidade') this.tipoCons = null;
    else this.tipoCons = '9';
    if (!modal) {
      this.modalController.createModal('modals/gridVigencias').then(() => {
        this.modalController.openModal();
        this.setFilterVigencia();
      });
    } else {
      this.modalController.openModal();
      this.setFilterVigencia();
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDADE',
      },
      {
        column: 'IDCNTRESTOQ',
        columnOnGrid: 'IDCNTRESTOQ',
      },
      {
        column: 'IDPRODATIV',
        columnOnGrid: 'IDPRODATIV',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/prodAgregAtivo',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };

  public getFilterProd() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
    ];
  }
  
  // Form export
  get controlExportButton() {
    const gridConsPadrao = Metadata.getInstance<GridEditable>('CONSPADRAO');
    if (gridConsPadrao.selectedRows.length === 0) return true;
    return false;
  }
  
  public async createModalExport() {
    this.modalExport = await ModalService.createFromJson('modals/formExportConsumoServ', true);
  }
  
  public openExportModal() {
    const gridConsPadrao = Metadata.getInstance<GridEditable>('CONSPADRAO');
    if (gridConsPadrao.selectedRows.length !== 0) {
      this.createModalExport().then(() => {
        this.modalExport.show();
      });
    }
  }
  
  public cancelModal() {
    this.modalExport.hide();
    this.modalExport.destroy();
  }
  
  public setFilterUnidade({ component }: IEventParam<SelectMultiple>) {
    const formExport = Metadata.getInstance<Form>('formExportConsumoServ');
    const central = formExport.value.CENTRAL;
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central,
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServico({ component }: IEventParam<SelectMultiple>) {
    const unidade = Metadata.getInstance<SelectMultiple>('UNIDADE_EXPORT');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';

    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public clearFieldsExport({ component }: IEventParam<SelectMultiple>) {
    const unidade = Metadata.getInstance<SelectMultiple>('UNIDADE_EXPORT');
    const servico = Metadata.getInstance<SelectMultiple>('SERVICO_EXPORT');
    
    if (component.name === 'CENTRAL') {
      unidade.setValue('');
      servico.setValue('');
    }
  }
  
  public setValuesExport() {
    const formCons = Metadata.getInstance<Form>('formCons');
    const consumoFormCons = formCons.value.TPCONSUMO;
    const IDQTDECONS = Metadata.getInstance<Select>('IDQTDECONS');
    const selectCentral = Metadata.getInstance<Select>('CENTRAL');
    selectCentral.setValue(this.appController.getCdCentral());
    if (consumoFormCons === 'F') IDQTDECONS.value = 'F';
    else IDQTDECONS.value = 'P';
  }
  
  public validaData() {
    const formExport = Metadata.getInstance<Form>('formExportConsumoServ');
    const row: IDictionary = formExport.value;
    const dataComponente = moment(row.DTCONSSERVGRAVA, 'DD/MM/YYYY').format('dddd');
    const fieldDiaSemana = Metadata.getInstance<TextInput>('DIASEMANA');
    fieldDiaSemana.value = dataComponente;
  }
  
  public returnDiaSemana(data: string) {
    const semana = ['Domingo', 'Segunda-Feira', 'Terça-Feira', 'Quarta-Feira', 'Quinta-Feira', 'Sexta-Feira', 'Sábado'];
    let result = '';
    if (data !== null) {
      const arr: IDictionary = data.split('/').reverse();
      const dateFormat = new Date(arr[0], arr[1] - 1, arr[2]);
      const dia = dateFormat.getDay();
      result = semana[dia];
    }

    return result;
  }
  
  public exportEfetivosConsumo({ component }: IEventParam<Button>) {
    const formExport = Metadata.getInstance<Form>('formExportConsumoServ');
    const gridConsPadrao = Metadata.getInstance<GridEditable>('CONSPADRAO');
    const row = formExport.value;
    const dtCons = moment(row.DTCONSSERVGRAVA, 'DD/MM/YYYY');
    const rowGrid = gridConsPadrao.selectedRows;
    row.PRODUTOS = rowGrid;
    
    if (formExport.validate()) {
      if (row.IDTIPOCONSGRAVA.length > 1) {
        this.utilities.activeLoadingButton(component);
        row.DTCONSSERVGRAVA = dtCons.format('DD/MM/YYYY');
        Http.post('/insertListaExport', { requestType: 'Row', row }).then(() => {
          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
        }).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      } else this.utilities.showAlert('Alert', 'pme::selecioneDiaConsumo', 'error');
    }
  }
  
  // funcoes do modal cad. automatico excecao
  
  public async createModalCadAut() {
    this.modalCadAut = await ModalService.createFromJson('modals/pme00706_cadastramento_automatico_excecoes', true);
  }
  
  public openExportModalCadAut() {
    this.createModalCadAut().then(() => {
      this.modalCadAut.show();
    });
  }
  
  public cancelModalCadAut() {
    this.modalCadAut.hide();
    this.modalCadAut.destroy();
  }
  
  public validaDataCadAut() {
    const formCadAut = Metadata.getInstance<Form>('formCadAutExcecoes');
    const row: IDictionary = formCadAut.value;
    const dtDe = row.DTDE;
    const dtAte = row.DTATE;
    const diaSemanaDe = Metadata.getInstance<TextInput>('DIASEMANADE');
    const diaSemanaAte = Metadata.getInstance<TextInput>('DIASEMANAATE');
    
    const checkDom = Metadata.getInstance<CheckboxMultiple>('checkbox-dom');
    const checkSeg = Metadata.getInstance<CheckboxMultiple>('checkbox-seg');
    const checkTer = Metadata.getInstance<CheckboxMultiple>('checkbox-ter');
    const checkQua = Metadata.getInstance<CheckboxMultiple>('checkbox-qua');
    const checkQui = Metadata.getInstance<CheckboxMultiple>('checkbox-qui');
    const checkSex = Metadata.getInstance<CheckboxMultiple>('checkbox-sex');
    const checkSab = Metadata.getInstance<CheckboxMultiple>('checkbox-sab');
    
    if (dtDe) diaSemanaDe.value = this.returnDiaSemana(dtDe);
    if (dtAte) diaSemanaAte.value = this.returnDiaSemana(dtAte);
    
    if (dtDe && dtAte) {
      if (this.utilities.dateIsValid(dtDe, dtAte)) {
        this.disableDiaSemana();
      } else {
        this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
        checkDom.disabled = true;
        checkSeg.disabled = true;
        checkTer.disabled = true;
        checkQua.disabled = true;
        checkQui.disabled = true;
        checkSex.disabled = true;
        checkSab.disabled = true;
      }
    }
  }
  
  public disableDiaSemana() {
    const checkDom = Metadata.getInstance<CheckboxMultiple>('checkbox-dom');
    const checkSeg = Metadata.getInstance<CheckboxMultiple>('checkbox-seg');
    const checkTer = Metadata.getInstance<CheckboxMultiple>('checkbox-ter');
    const checkQua = Metadata.getInstance<CheckboxMultiple>('checkbox-qua');
    const checkQui = Metadata.getInstance<CheckboxMultiple>('checkbox-qui');
    const checkSex = Metadata.getInstance<CheckboxMultiple>('checkbox-sex');
    const checkSab = Metadata.getInstance<CheckboxMultiple>('checkbox-sab');

    const formCadAut = Metadata.getInstance<Form>('formCadAutExcecoes');
    const row: IDictionary = formCadAut.value;
    let dtDe = moment(row.DTDE, 'DD/MM/YYYY');
    const dtAte = moment(row.DTATE, 'DD/MM/YYYY');
    const diferencaDias = dtAte.diff(dtDe, 'days');

    const result = [];
    
    if (row.DTDE && row.DTATE) {
      if (diferencaDias <= 7) {
        checkDom.disabled = true;
        checkSeg.disabled = true;
        checkTer.disabled = true;
        checkQua.disabled = true;
        checkQui.disabled = true;
        checkSex.disabled = true;
        checkSab.disabled = true;
        while (dtAte.diff(dtDe, 'days') >= 0) {
          result.push(dtDe.day() + 1);
          dtDe = dtDe.add(1, 'day');
        }
        result.forEach((posicao) => {
          if (posicao === 1) checkDom.disabled = false;
          if (posicao === 2) checkSeg.disabled = false;
          if (posicao === 3) checkTer.disabled = false;
          if (posicao === 4) checkQua.disabled = false;
          if (posicao === 5) checkQui.disabled = false;
          if (posicao === 6) checkSex.disabled = false;
          if (posicao === 7) checkSab.disabled = false;
        });
      } else {
        checkDom.disabled = false;
        checkSeg.disabled = false;
        checkTer.disabled = false;
        checkQua.disabled = false;
        checkQui.disabled = false;
        checkSex.disabled = false;
        checkSab.disabled = false;
      }
    }
  }
  
  public confirmarCadAut({ component }: IEventParam<Button>) {
    const formCadAut = Metadata.getInstance<Form>('formCadAutExcecoes');
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const dtExcecao = Metadata.getInstance<IDate>('DTEXCECAO');
    const row = formCadAut.value;
    row.filtro = formFiltro.value;
    row.CDFILCENTPME = this.appController.getCdCentral();
    
    const dom = Metadata.getInstance<CheckboxMultiple>('checkbox-dom').value.length;
    const seg = Metadata.getInstance<CheckboxMultiple>('checkbox-seg').value.length;
    const ter = Metadata.getInstance<CheckboxMultiple>('checkbox-ter').value.length;
    const qua = Metadata.getInstance<CheckboxMultiple>('checkbox-qua').value.length;
    const qui = Metadata.getInstance<CheckboxMultiple>('checkbox-qui').value.length;
    const sex = Metadata.getInstance<CheckboxMultiple>('checkbox-sex').value.length;
    const sab = Metadata.getInstance<CheckboxMultiple>('checkbox-sab').value.length;
    
    if (formCadAut.validate()) {
      if (dom !== 0 || seg !== 0 || ter !== 0 || qua !== 0 || qui !== 0 || sex !== 0 || sab !== 0) {
        if (row.IDTPCONSPME.length !== 0) {
          this.utilities.activeLoadingButton(component);
          Http.post('/cadastroDiaConsumo', { requestType: 'Row', row }).then(() => {
            this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
            this.cancelModalCadAut();
            this.filterDefaultDates(dtExcecao, '9');
          }).finally(() => {
            this.utilities.disableLoadingButton(component);
          });
        } else this.utilities.showAlert('Alert', 'pme::tipoConsumoAExcluirInfo', 'error');
      } else this.utilities.showAlert('Alert', 'pme::diaDaSemanaInformado', 'error');
    }
  }
  
  // Funções modal susbstituir produtos
  
  public openModalSubsProd() {
    this.createModalSubsProd().then(() => {
      this.modalSubsProd.show();
    });
  }
  
  public async createModalSubsProd() {
    this.modalSubsProd = await ModalService.createFromJson('modals/pme00707_substituir_produto', true);
  }
  
  get controlSubsProdButtonConsumo() {
    const gridConsPadrao = Metadata.getInstance<GridEditable>('CONSPADRAO');
    let result;
    if (gridConsPadrao.datasource.currentRow.CDARVPROD) result = false;
    else result = true;
    return result;
  }
  
  public setValuesModalSubsProd() {
    const formSubsProdOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const gridConsumo = Metadata.getInstance<GridEditable>('CONSPADRAO');
    const prodOrigem = Metadata.getInstance<Select>('PRODINICIAL');
    const row = formSubsProdOrigem.value;
    const rowGrid = gridConsumo.datasource.currentRow;
    const prodIni = {
      CDARVPROD: rowGrid.CDARVPROD,
      NMPRODUTO: rowGrid.NMPRODUTO,
      CDPRODUTO: rowGrid.CDPRODUTO,
    };
    prodOrigem.setValue(prodIni);
    row.DTREFERENCIA = rowGrid.DTCONSERVPME;
  }
  
  public cancelModalSubsProd() {
    const gridConsumo = Metadata.getInstance<GridEditable>('CONSPADRAO');
    this.modalSubsProd.hide();
    this.modalSubsProd.destroy();
    this.gridEditController.reload({ component: gridConsumo });
  }
  
  public filterAgregada({ component }: IEventParam<Select>) {
    component.datasource.filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    component.datasource.get();
  }
  
  public filterAgregadaSubsProd() {
    const formSubsProdOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const row = formSubsProdOrigem.value;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NRLISTA = '';
  
    return Http.post('/subsProd/buscaUnidade', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result.data;
      row.NRLISTA = result;
      return row.NRLISTA;
    });
  }
  
  public setFilterServicoSubsProd(row: IDictionary) {
    const servico = Metadata.getInstance<SelectMultiple>('SERVICOSUBS');
    const central = row.CDFILCENTPME;
    const filter = [
      {
        name: 'P_CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: central,
      },
      {
        name: 'P_NRLISTA',
        operator: '=',
        value: row.NRLISTA,
      },
    ];
    servico.datasource.filter = filter;
    servico.datasource.get();
  }
  
  public setServicoSubsProd() {
    const formSubsProdOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const row: IDictionary = formSubsProdOrigem.value;
    
    this.filterAgregadaSubsProd().then((result) => {
      row.NRLISTA = result;
      this.setFilterServicoSubsProd(row);
    });
  }
  
  public setGridSubsProd({ component }: IEventParam<Select>) {
    const formSubsProdOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const servico = Metadata.getInstance<SelectMultiple>('SERVICOSUBS');
    const row = formSubsProdOrigem.value;
    
    if (component.name === 'UNIDADE') servico.setValue('');
    
    if (row.NRLISTA === undefined) {
      this.filterAgregadaSubsProd().then((result) => {
        row.NRLISTA = result;
        this.setFilterGridSubsProd(row);
      });
    } else this.setFilterGridSubsProd(row);
  }
  
  public setFilterGridSubsProd(row: IDictionary) {
    const formSubsProdOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const gridSubsProd = Metadata.getInstance<GridEditable>('gridSubsProd');
    row.LISTASERVFILI = '';
    row.CDFILCENTPME = this.appController.getCdCentral();
    const tipoCons = row.IDQTDECONSUB;
    if (tipoCons.length > 0) {
      if (formSubsProdOrigem.validate()) {
        Http.post('/subsProd/buscaServico', { requestType: 'Row', row }).then((response) => {
          const result = response.data.dataset.result.data;
          row.LISTASERVFILI = result;
          const filter = [
            {
              name: 'P_CDPRODUTO',
              operator: '=',
              value: formSubsProdOrigem.value.PRODINICIAL.CDPRODUTO,
            },
            {
              name: 'P_DTCONSSERV',
              operator: '=',
              value: formSubsProdOrigem.value.DTREFERENCIA,
            },
            {
              name: 'P_IDTIPOCONPME',
              operator: '=',
              value: formSubsProdOrigem.value.DIASCONSUMO.toString(),
            },
            {
              name: 'P_LFILISERV',
              operator: '=',
              value: result,
            },
            {
              name: 'P_IDQTDECONS',
              operator: '=',
              value: tipoCons,
            },
          ];
          
          gridSubsProd.datasource.filter = filter;
          gridSubsProd.datasource.get().then(() => {
            this.setValuesGridSubsProd(gridSubsProd);
          });
        });
      } else gridSubsProd.datasource.data = [];
    } else this.utilities.showAlert('alertSuccess', 'pme::tipoConsumoInfo', 'warning');
  }
  
  public setValuesGridSubsProd(gridSubsProd: GridEditable) {
    const data = gridSubsProd.datasource.data;
    if (data.length > 0) {
      data.forEach((itens) => {
        if (itens.IDQTDECONPME === 'F') itens.IDQTDECONPME = I18n.translate('pme::fixo');
        else if (itens.IDQTDECONPME === 'P') itens.IDQTDECONPME = I18n.translate('pme::perCapita');
        else itens.IDQTDECONPME = I18n.translate('pme::ambos');
        if (itens.IDTIPOCONPME === '1') itens.IDTIPOCONPME = I18n.translate('pme::domingo');
        else if (itens.IDTIPOCONPME === '2') itens.IDTIPOCONPME = I18n.translate('pme::segunda');
        else if (itens.IDTIPOCONPME === '3') itens.IDTIPOCONPME = I18n.translate('pme::terca');
        else if (itens.IDTIPOCONPME === '4') itens.IDTIPOCONPME = I18n.translate('pme::quarta');
        else if (itens.IDTIPOCONPME === '5') itens.IDTIPOCONPME = I18n.translate('pme::quinta');
        else if (itens.IDTIPOCONPME === '6') itens.IDTIPOCONPME = I18n.translate('pme::sexta');
        else if (itens.IDTIPOCONPME === '7') itens.IDTIPOCONPME = I18n.translate('pme::sabado');
        else if (itens.IDTIPOCONPME === '8') itens.IDTIPOCONPME = I18n.translate('pme::feriado');
      });
    }
  }
  
  public changeProdOrigem() {
    const formOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const prodOrigem = Metadata.getInstance<Select>('PRODINICIAL');
    const row: IDictionary = formOrigem.value;
    row.CDFILCENTPME = this.appController.getCdCentral();
    if (this.reportController.controllProductIni) {
      Http.post('/subsProd/validaProdUnid', { requestType: 'Row', row }).then((response) => {
        const result = response.data.dataset.result.data;
        if (result.length > 0 && result[0].IDPRODATIV === 'A') {
          if (result[0].IDPRODATIVPR !== 'S') {
            prodOrigem.setValue(null);
            this.utilities.showAlert('alertSuccess', 'pme::prodInativoUnidade', 'error');
          }
        } else {
          prodOrigem.setValue(null);
          this.utilities.showAlert('alertSuccess', 'pme::produtoOrigemInativo', 'error');
        }
      });
    }
  }
  
  public changeProdFinal() {
    const formDestino = Metadata.getInstance<Form>('formSubsProdDestino');
    const prodDestino = Metadata.getInstance<Select>('PRODFINAL');
    const row: IDictionary = formDestino.value;
    row.CDFILCENTPME = this.appController.getCdCentral();
    Http.post('/subsProd/validaProdUnid', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result.data;
      if (result.length > 0 && result[0].IDPRODATIV === 'A') {
        if (result[0].IDPRODATIVPR !== 'S') {
          prodDestino.setValue(null);
          this.utilities.showAlert('alertSuccess', 'pme::prodInativoUnidade', 'error');
        }
      } else {
        prodDestino.setValue(null);
        this.utilities.showAlert('alertSuccess', 'pme::produtoDestinoInativo', 'error');
      }
    });
  }
  
  public confirmarSubsProd() {
    const formOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const formDestino = Metadata.getInstance<Form>('formSubsProdDestino');
    const gridSubsProd = Metadata.getInstance<Grid>('gridSubsProd');
    
    if ((formDestino.validate()) && (formOrigem.validate())) {
      if (gridSubsProd.selectedRows.length > 0) {
        this.confirmAction();
      } else this.utilities.showAlert('alertSuccess', 'pme::nenhumRegSelecionado', 'warning');
    }
  }
  
  public confirmAction() {
    const grid = Metadata.getInstance<Grid>('gridSubsProd');
    this.selectedLine = grid.selectedRows;
    if (this.selectedLine.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pme_warningSubsProd',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::confirmaSubsProd',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.insertSubsProd(this.selectedLine) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public insertSubsProd(checkedRows: IDictionary) {
    const formOrigem = Metadata.getInstance<Form>('formSubsProdOrigem');
    const formDestino = Metadata.getInstance<Form>('formSubsProdDestino');
    const gridSubsProd = Metadata.getInstance<GridEditable>('gridSubsProd');
    const rowOrigem = formOrigem.value;
    const rowDestino = formDestino.value;
    
    checkedRows.forEach((item: IDictionary) => {
      item.CDFILCENTPME = rowOrigem.CDFILCENTPME;
      item.CDPRODDEST = rowDestino.PRODFINAL;
      item.CDPRODORIG = rowOrigem.PRODINICIAL;
      item.DTCONSERV = rowOrigem.DTREFERENCIA;
      item.SERVICOSUBS = rowOrigem.SERVICOSUBS;
      item.UNIDADE = rowOrigem.UNIDADE;
      item.NRLISTA = rowOrigem.NRLISTA;
      item.TIPOLOG = rowOrigem.DIASCONSUMO;
      item.QTLOG = rowOrigem.IDQTDECONSUB;
      if (item.IDQTDECONPME === 'Fixo' || item.IDQTDECONPME === 'Fixed') item.IDQTDECONPME = 'F';
      else if (item.IDQTDECONPME === 'Per Capita') item.IDQTDECONPME = 'P';
      else if (item.IDQTDECONPME === 'Ambos' || item.IDQTDECONPME === 'Both') item.IDQTDECONPME = 'A';
      if (item.IDTIPOCONPME === 'Domingo' || item.IDTIPOCONPME === 'Sunday') item.IDTIPOCONPME = '1';
      else if (item.IDTIPOCONPME === 'Segunda-feira' || item.IDTIPOCONPME === 'Monday') item.IDTIPOCONPME = '2';
      else if (item.IDTIPOCONPME === 'Terça-feira' || item.IDTIPOCONPME === 'Tuesday') item.IDTIPOCONPME = '3';
      else if (item.IDTIPOCONPME === 'Quarta-feira' || item.IDTIPOCONPME === 'Wednesday') item.IDTIPOCONPME = '4';
      else if (item.IDTIPOCONPME === 'Quinta-feira' || item.IDTIPOCONPME === 'Thursday') item.IDTIPOCONPME = '5';
      else if (item.IDTIPOCONPME === 'Sexta-feira' || item.IDTIPOCONPME === 'Friday') item.IDTIPOCONPME = '6';
      else if (item.IDTIPOCONPME === 'Sábado' || item.IDTIPOCONPME === 'Saturday') item.IDTIPOCONPME = '7';
      else if (item.IDTIPOCONPME === 'Feriado' || item.IDTIPOCONPME === 'Holiday') item.IDTIPOCONPME = '8';
    });
    
    Http.post('/subsProd/insertSubsProd', { checkedRows }).then(() => {
      DialogService.hide();
      this.utilities.showAlert('save', 'pme::salvoComSucesso', 'green');
      this.setValuesGridSubsProd(gridSubsProd);
    });
  }
}
