import {
  Http, IDictionary, Loader, IEventParam, Metadata,
} from '@zeedhi/core';
import { Select, Form } from '@zeedhi/common';

export class ReportController {
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');

  public generateReport(url: string, row: IDictionary, cdReport: string) {
    row.__report_name = cdReport;
    return Http.post(url, { requestType: 'Row', row }).then((response) => {
      const uri = response.data.messages[0].message;
      this.openReport(uri);
    });
  }

  private openReport(uri: string) {
    const link = document.createElement('a');
    link.download = uri;
    link.href = uri;
    link.target = '_blank';
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    let agrupUnidValue = 'T';
    if (agrupUnid.value != null) {
      if (typeof agrupUnid.value === 'object') agrupUnidValue = agrupUnid.value.CDGRUPFILI;
      else agrupUnidValue = agrupUnid.value;
    }
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterAgreg({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterAgregTpCard({ component }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');

    let tpCardValue = '';
    if (tipoCard.returnObject) tpCardValue = tipoCard.value ? tipoCard.value.CDTIPOCARD : 'T';
    else tpCardValue = tipoCard.value ? tipoCard.value : 'T';
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tpCardValue,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPOSERVICO');
    const grupServValue = grupserv.value ? grupserv.value : 'T';
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    let agrupUnidValue = 'T';
    if (agrupUnid.value != null) {
      if (typeof agrupUnid.value === 'object') agrupUnidValue = agrupUnid.value.CDGRUPFILI;
      else agrupUnidValue = agrupUnid.value;
    }

    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setFilterServ({ component }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    let tpCardValue = '';
    if (tipoCard.returnObject) tpCardValue = tipoCard.value ? tipoCard.value.CDTIPOCARD : '';
    else if (Array.isArray(tipoCard.value)) tpCardValue = tipoCard.value.join('_');
    else tpCardValue = tipoCard.value ? tipoCard.value : '';

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tpCardValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public clearFields({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');
    this.utilities.clearPeriodoParams();

    if (component.name === 'TIPOCARD' || component.name === 'GRUPFILI') {
      servico.setValue('');
      unidade.setValue('');
    } else if (component.name === 'UNIDADE' || component.name === 'GRUPOSERVICO') {
      servico.setValue('');
    } else unidade.setValue('');
  }
  
  public formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (this.utilities.centralIsValid() && form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        valid = true;
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }
  
  public verifyProductIni({ component }: IEventParam<Select>) {
    const productIni = Metadata.getInstance<Select>('PRODINICIAL').value.CDARVPROD;
    const productFin = Metadata.getInstance<Select>('PRODFINAL').value.CDARVPROD;
    
    if (productIni > productFin) {
      this.utilities.showAlert('Alert', 'pme::prodIniMaior', 'error');
      component.setValue(null);
    }
  }
  
  public controllProductIni() {
    const productFin = Metadata.getInstance<Select>('PRODFINAL');
    if (productFin.value !== null) productFin.setValue(null);
  }

  public defaultTpCusto({ component }: IEventParam<Select>) {
    const language = localStorage.getItem('LANGUAGE') as string;
    let idioma = null;
    if (language != null && language.includes('en')) idioma = 'I';
    else if (language != null && language.includes('es')) idioma = 'E';
    
    const conditions = [
      {
        name: 'STIDIOMA',
        operator: '=',
        value: idioma,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get().then((response: IDictionary) => {
      if (response.length > 0) component.setValue(response[0]);
    });
  }
}
